
### SATISFACTORY RESPONSE
## Not found code
OK = 200
OK_NOCONTENT = 204

### CLIENT ERROR
## Not found code
NOT_FOUND = 404
UNAUTHORIZED = 401

### SERVER ERROR
## Internal server error
INTERNAL_ERROR = 500




import subprocess

class Request:
    def __init__(self, url="", headers={}, body={}):
        self.__headers = headers
        self.__body = body
        self.__url = url

    def getUrl(self):
        return self.__url

    def getHeaders(self):
        return self.__headers

    def geyBody(self):
        return self.__body

    def __getHeadersRequest(self):
        result = []
        for key in self.__headers:
            result.append("-H")
            result.append("'{}: {}'".format(key, self.__headers[key]))
        return result

    def __getBodyRequest(self):
        result = "'{}'".format(str(self.__body).replace("'", '"'))
        return result

    def GET(self):
        try:
            curl = ["curl", "-XGET"] + self.__getHeadersRequest() + [self.__url]
            result = subprocess.check_output(' '.join(curl), shell=True, stderr=subprocess.STDOUT)
        except:
            result = None
        return result

    def POST(self):
        try:
            curl = ["curl", "-XPOST", "-d", self.__getBodyRequest()] + self.__getHeadersRequest() + [self.__url]
            result = subprocess.check_output(' '.join(curl), shell=True, stderr=subprocess.STDOUT)
        except:
            result = None
        return result

    def PUT(self):
        try:
            curl = ["curl", "-XPUT", "-d", self.__getBodyRequest()] + self.__getHeadersRequest() + [self.__url]
            result = subprocess.check_output(' '.join(curl), shell=True, stderr=subprocess.STDOUT)
        except:
            result = None
        return result

    def DELETE(self):
        try:
            curl = ["curl", "-XDELETE"] + self.__getHeadersRequest() + [self.__url]
            result = subprocess.check_output(' '.join(curl), shell=True, stderr=subprocess.STDOUT)
        except:
            result = None
        return result




FORMAT = {
	"instance": {
		"hostName": "",
		"app": "",
		"instanceId": "",
		"vipAddress": "",
		"secureVipAddress": "",
		"ipAddr": "",
		"status": "",
		"port": {"$": "", "@enabled": ""},
		"securePort": {"$": "", "@enabled": ""},
		"healthCheckUrl": "",
		"statusPageUrl": "",
		"homePageUrl": "",
		"dataCenterInfo": {
			"@class": "",
			"name": ""
		}
	}
}




from eurekaClientShare.client.instance import Instance
from eurekaClientShare.util.request import Request
import eurekaClientShare.constant.http_code as CODE
import time
from threading import Thread

class DiscoveryClient:
    def __init__(self, info):
        self.__info = info
        self.__instance = Instance(self.__info)
        self.__beat = int(info['HEARTBEATDISCOVER'])
        self.__beats = 0
        self.__url = "{}/{}".format(info['HOSTDISCOVER'], self.__instance.getAppName())
        self.headers = {"Content-Type": "application/json"}
        self.__create()
        self.__instance.setState("UP", self.__info)
        self.startHeartbeat()


    def __create(self):
        urlDelete = "{}/{}".format(self.__url, self.__instance.getInstanceId())
        Request(url=urlDelete, headers=self.headers).DELETE()
        result = Request(url=self.__url, body=self.__instance.getDict(), headers=self.headers).POST()
        return result

    def __heartbeat(self):
        while self.__instance.getStatus() == "UP":
            time.sleep(self.__beat)
            self.__send()
        return

    def __send(self):
        self.__beats += 1
        urlInstance = "{}/{}/status?value={}".format(self.__url, self.__instance.getInstanceId(), self.__instance.getStatus())
        result = Request(url=urlInstance, headers=self.headers).PUT()
        return result

    def startHeartbeat(self):
        self.__heart = Thread(target=self.__heartbeat)
        self.__heart.daemon = True
        self.__heart.start()
        return

    def stop(self):
        self.__instance.setState("DOWN", self.__info)
        self.__send()
        return 1




from eurekaClientShare.constant.format import FORMAT
import json

class Instance:
    def __init__(self, info):
        self.__format = FORMAT
        self.__state = "STARTING"
        self.__set(info)

    def __set(self, info):
        self.__format["instance"]["hostName"] = info['HOSTINSTANCE']
        self.__format["instance"]["app"] = info['SERVICENAME']
        self.__format["instance"]["instanceId"] = "{}:{}:{}".format(info['HOSTINSTANCE'], info['SERVICENAME'], info['PORTINSTANCE'])
        self.__format["instance"]["vipAddress"] = "{}{}:{}".format(info['PROTOCOL'], info['HOSTINSTANCE'], info['PORTINSTANCE'])
        self.__format["instance"]["secureVipAddress"] = self.__format["instance"]["vipAddress"]
        self.__format["instance"]["ipAddr"] = self.__format["instance"]["vipAddress"]
        self.__format["instance"]["status"] = self.__state
        self.__format["instance"]["port"] = {"$": info['PORTINSTANCE'], "@enabled": "true"}
        self.__format["instance"]["securePort"] = {"$": info['PORTINSTANCE'], "@enabled": "true"}
        self.__format["instance"]["healthCheckUrl"] = "{}{}:{}/healthcheck".format(info['PROTOCOL'], info['HOSTINSTANCE'], info['PORTINSTANCE'])
        self.__format["instance"]["statusPageUrl"] = "{}{}:{}/status".format(info['PROTOCOL'], info['HOSTINSTANCE'], info['PORTINSTANCE'])
        self.__format["instance"]["homePageUrl"] = "{}{}:{}".format(info['PROTOCOL'], info['HOSTINSTANCE'], info['PORTINSTANCE'])
        self.__format["instance"]["dataCenterInfo"] = {
			"@class": "com.netflix.appinfo.InstanceInfo$DefaultDataCenterInfo",
			"name": "MyOwn"
		}

    def setState(self, state, info):
        self.__state = state
        self.__set(info)
        return self

    def getStatus(self):
        return self.__state

    def getAppName(self):
        return self.__format["instance"]["app"]

    def getInstanceId(self):
        return self.__format["instance"]["instanceId"]

    def getDict(self):
        return self.__format

    def getString(self):
        return str(self.__format).replace("'", '"')

    def getJson(self):
        result = str(self.__format).replace("'", '"')
        return json.dumps(result)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="eurekaClientShare",
    version="0.1.1",
    author="Santiago Melo Medina",
    author_email="santiagodevelopment001@gmail.com",
    description="Simple eureka client",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/santiagoMeloMedina/eurekaClientShare",
    packages=["eurekaClientShare/asset", "eurekaClientShare/client", "eurekaClientShare/constant", "eurekaClientShare/util"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
    ],
    python_requires='>=3.6'
)
